'-----------------------------------------------------------
'Quellprogramm: JULIA3.BAS
'-----------------------------------------------------------
'Das Programm berechnet die Julia-Menge fuer die
'Iteration z(n+1) = z(n)^2 + c nach dem
'Level-Set-Algorithmus, ausserdem alle Fixpunkte
'der 1. und 2. Iterierten mit expliziten Formeln.
'Anziehende Fixpunkte mit hoeherer Periodenzahl
'werden numerisch angenaehert.
'-----------------------------------------------------------
'Eingabewerte:
a = -.1                  'Parameter c = (a, b)
b = .75
nmax = 100             'Anzahl der Iterationen nmax
'-----------------------------------------------------------

SCREEN 12              'VGA, 640 x 480 Pixel, 16 Farben
px = 420: py = 420     'Bildschirmbereich

'z-Ebene                            'mathematischer Bereich
xmin = -2: xmax = 2                 'Realteil
ymin = -2: ymax = 2                 'Imaginaerteil

xschritt = (xmax - xmin) / px       'Schrittweite
yschritt = (ymax - ymin) / py

'Fluchtschranke
r = 2 + SQR(a ^ 2 + b ^ 2)
r2 = r ^ 2

CLS

'Berechnung der ausgefuellten Julia-Menge durch
'Test auf Mengenzugehoerigkeit:
x = 0: y = 0
FOR sx = 0 TO px                    'Bildschirmspalten
   x = xmin + sx * xschritt
   FOR zy = 0 TO py                 'Bildschirmzeilen
      y = ymax - zy * yschritt
      x1 = x: y1 = y
      FOR n = 1 TO nmax                 'Iteration
         xx = x1 * x1 - y1 * y1 + a     'Hilfsgroesse
         y1 = 2 * x1 * y1 + b           'Imaginaerteil
         x1 = xx                        'Realteil
         IF x1 * x1 + y1 * y1 > r2 THEN EXIT FOR
      NEXT n
      'Punktausgabe
      IF x1 * x1 + y1 * y1 <= r2 THEN PSET (sx, zy), 9
   NEXT zy
NEXT sx

'Fixpunkte der 1. Iterierten
p = 1 - 4 * a: q = -4 * b
u = SQR((p + SQR(p ^ 2 + q ^ 2)) / 2)
IF u = 0 THEN v = SQR(-p) ELSE v = q / 2 / u
x1 = (1 + u) / 2: y1 = v / 2
x2 = (1 - u) / 2: y2 = -v / 2
sx1 = (x1 - xmin) / xschritt: zy1 = (ymax - y1) / yschritt
sx2 = (x2 - xmin) / xschritt: zy2 = (ymax - y2) / yschritt
CIRCLE (sx1, zy1), 3, 10: CIRCLE (sx2, zy2), 3, 10

'Fixpunkte der 2. Iterierten
p = -3 - 4 * a: q = -4 * b
u = SQR((p + SQR(p ^ 2 + q ^ 2)) / 2)
IF u = 0 THEN v = SQR(-p) ELSE v = q / 2 / u
x1 = -(1 + u) / 2: y1 = -v / 2
x2 = -(1 - u) / 2: y2 = v / 2
sx1 = (x1 - xmin) / xschritt: zy1 = (ymax - y1) / yschritt
sx2 = (x2 - xmin) / xschritt: zy2 = (ymax - y2) / yschritt
CIRCLE (sx1, zy1), 3, 12: CIRCLE (sx2, zy2), 3, 12

'Iteration zur naeherungsweisen Fixpunktberechnung
x = 0: y = 0
FOR i = 1 TO 1000
   xx = x * x - y * y + a
   y = 2 * x * y + b
   x = xx
   IF x * x + y * y > r2 THEN EXIT FOR
   sx = (x - xmin) / xschritt
   zy = (ymax - y) / yschritt
   IF i > 900 THEN CIRCLE (sx, zy), 3
NEXT i

LOCATE 28
PRINT "Berechnungsparameter: a = "; a; ";  b = "; b; ";  n max = "; nmax

END



